import {
  ChangeDetectionStrategy,
  Component,
  Input,
} from "@angular/core";
import {OrganizationalUnit} from "@models";
import {Store} from "@ngxs/store";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {SharedOrgUnitAction} from "@shared/stores/organizational-unit/shared-organizational-unit.action";
import {SharedOrganizationalUnitState} from "@shared/stores/organizational-unit/shared-organizational-unit.state";
import {Observable} from "rxjs";
import {map} from "rxjs/operators";
import {
  isNullOrUndefined,
  ResourceState,
  StringUtil,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-organizational-unit-name-container",
  templateUrl: "./shared-organizational-unit-name.container.html",
  styleUrls: ["./shared-organizational-unit-name.container.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedOrganizationalUnitNameContainer extends SharedAbstractContainer {
  name: Observable<string> = ResourceState.list(this._store, SharedOrganizationalUnitState).pipe(
    map(list => this.mapOrgUnitName(list)),
  );

  _id: string;

  @Input()
  set id(id: string) {
    if (!isNullOrUndefined(id)) {
      this._store.dispatch(new SharedOrgUnitAction.AddInListById(id, true));
    }
    this._id = id;
  }

  get id(): string {
    return this._id;
  }

  constructor(public readonly _store: Store) {
    super();
  }

  private mapOrgUnitName(list: OrganizationalUnit[]): string {
    if (isNullOrUndefined(list)) {
      return StringUtil.stringEmpty;
    }
    const organizationalUnit = list.find(o => o.resId === this._id);
    if (isNullOrUndefined(organizationalUnit)) {
      return StringUtil.stringEmpty;
    }
    return organizationalUnit.name;
  }
}
