import {
  ChangeDetectionStrategy,
  Component,
  Inject,
  OnInit,
} from "@angular/core";
import {MAT_DIALOG_DATA} from "@angular/material/dialog";
import {TranslateService} from "@ngx-translate/core";
import {Store} from "@ngxs/store";
import {SharedAbstractContainer} from "@shared/components/containers/shared-abstract/shared-abstract.container";
import {FieldTypeEnum} from "@shared/enums/field-type.enum";
import {DataTableColumns} from "@shared/models/data-table-columns.model";
import {StatusHistoryDialog} from "@shared/models/status-history-dialog.model";
import {
  OrderEnum,
  QueryParameters,
  StringUtil,
  TRANSLATE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-history-dialog",
  templateUrl: "./shared-history.dialog.html",
  styleUrls: ["./shared-history.dialog.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedHistoryDialog extends SharedAbstractContainer implements OnInit {
  paramMessage: { name: string } = {name: ""};
  columns: DataTableColumns[];

  constructor(private translate: TranslateService,
              private store: Store,
              @Inject(MAT_DIALOG_DATA) public data: StatusHistoryDialog) {
    super();
    this.paramMessage.name = StringUtil.convertToPascalCase(this.data.name);
  }

  ngOnInit(): void {
    super.ngOnInit();
    this.columns = [
      {
        field: "changeTime",
        header: TRANSLATE("app.history.table.changeTime"),
        type: FieldTypeEnum.datetime,
        order: OrderEnum.descending,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "status",
        header: TRANSLATE("app.history.table.status"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isSortable: false,
        isFilterable: false,
        width: "110px",
      },
      {
        field: "description",
        header: TRANSLATE("app.history.table.description"),
        type: FieldTypeEnum.number,
        order: OrderEnum.none,
        isSortable: true,
        isFilterable: false,
      },
      {
        field: "createdBy",
        header: TRANSLATE("app.history.table.createdBy"),
        type: FieldTypeEnum.string,
        order: OrderEnum.none,
        isSortable: true,
        isFilterable: false,
        width: "140px",
      },
    ];
  }

  pagination($event: QueryParameters): void {
    this.store.dispatch(new this.data.state.ChangeQueryParameters($event, this.data.resourceResId, this.data.parentId));
  }
}
