import {
  ChangeDetectionStrategy,
  Component,
  Input,
  OnInit,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {environment} from "@environments/environment";
import {PackageStatusEnum} from "@shared/enums/business/package-status.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {AipCopyList} from "@shared/models/business/aip-copy-list.model";
import {AipCopy} from "@shared/models/business/aip-copy.model";
import {
  isNotNullNorUndefined,
  isNullOrUndefined,
  TRANSLATE,
  urlSeparator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-status-summary",
  templateUrl: "./aip-status-summary.presentational.html",
  styleUrls: ["./aip-status-summary.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipStatusSummaryPresentational extends SharedAbstractPresentational implements OnInit {
  @Input()
  aipCopyList: AipCopyList;
  aipCopySorted: AipCopy[] = [];

  private readonly PATH_TO_AIP_STORAGION: string = "#" + urlSeparator + AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aip + urlSeparator;

  get packageStatusEnum(): typeof PackageStatusEnum {
    return PackageStatusEnum;
  }

  ngOnInit(): void {
    super.ngOnInit();
    const storagionUrl: string[] = environment.storagionUrls.map(sto => sto.linkedTo);
    storagionUrl.forEach(url => {
      this.aipCopyList.copies.forEach(copy => {
        if (copy.storageUrl === url) {
          this.aipCopySorted.push(copy);
        }
      });
    });
  }

  getStatusLabel(status: PackageStatusEnum): string {
    if (isNullOrUndefined(status)) {
      return TRANSLATE("COPY_MISSING");
    }
    if (status === PackageStatusEnum.IN_PREPARATION) {
      return TRANSLATE("IN_PREPARATION");
    }
    if (status === PackageStatusEnum.REINDEXING) {
      return TRANSLATE("REINDEXING");
    }
    return status;
  }

  getStoragionNumber(aipCopy: AipCopy): number {
    const storagion = environment.storagionUrls.find(sto => aipCopy.storageUrl === sto.linkedTo).index;
    if (isNotNullNorUndefined(storagion)) {
      return storagion;
    }
    return 1;
  }

  getUrl(aipCopy: AipCopy): string | undefined {
    if (isNullOrUndefined(aipCopy.status)) {
      return undefined;
    }

    return this.PATH_TO_AIP_STORAGION + this.getStoragionNumber(aipCopy) + urlSeparator + PreservationPlanningRoutesEnum.aipDetail + urlSeparator + aipCopy.aip.resId;
  }
}
