import {
  AfterViewInit,
  Input,
  OnInit,
  Output,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  BaseResourceType,
  isEmptyString,
  isNullOrUndefined,
  ObservableUtil,
} from "solidify-frontend";

export abstract class SharedAbstractContentPresentational<TResource extends BaseResourceType> extends SharedAbstractPresentational implements OnInit, AfterViewInit {
  @Input()
  abstract host: any;

  private readonly _closeBS: BehaviorSubject<void> = new BehaviorSubject<void>(undefined);
  @Output("closeChange")
  readonly closeObs: Observable<void> = ObservableUtil.asObservable(this._closeBS);

  constructor() {
    super();
  }

  ngOnInit(): void {
    super.ngOnInit();
  }

  abstract select(value: TResource): void;

  close(): void {
    this._closeBS.next();
  }

  isActive(value: TResource): boolean {
    if (isNullOrUndefined(this.host.formControl.value) || isEmptyString(this.host.formControl.value)) {
      return false;
    }
    return this.host.formControl.value.includes(value[this.host.valueKey]);
  }
}



