import {CdkDragStart} from "@angular/cdk/drag-drop";
import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  Input,
  Output,
  ViewChild,
  ViewEncapsulation,
} from "@angular/core";
import {MatCheckboxChange} from "@angular/material/checkbox";
import {MatDialog} from "@angular/material/dialog";
import {MatSelect} from "@angular/material/select";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {SharedPaginatorPresentational} from "@app/shared/components/presentationals/shared-paginator/shared-paginator.presentational";
import {FieldTypeEnum} from "@app/shared/enums/field-type.enum";
import {DataTableColumns} from "@app/shared/models/data-table-columns.model";
import {environment} from "@environments/environment";
import {
  SharedDataTableLooseSelectionDialog,
  SharedDataTableLooseSelectionDialogData,
} from "@shared/components/dialogs/shared-data-table-loose-selection/shared-data-table-loose-selection.dialog";
import {DataTableComponentEnum} from "@shared/enums/data-table-component.enum";
import {DataTableActions} from "@shared/models/data-table-actions.model";
import {DataTableBulkActions} from "@shared/models/data-table-bulk-actions.model";
import {BreakpointService} from "@shared/services/breakpoint.service";
import {LazyLoadEvent} from "primeng/api";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {tap} from "rxjs/operators";
import {
  BaseResource,
  DateUtil,
  isEmptyArray,
  isEmptyString,
  isNullOrUndefined,
  isObservable,
  isTrue,
  MappingObject,
  MappingObjectUtil,
  NotificationService,
  ObjectUtil,
  ObservableUtil,
  OrderEnum,
  Paging,
  QueryParameters,
  QueryParametersUtil,
  Sort,
  StringUtil,
  TRANSLATE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-data-table",
  templateUrl: "./shared-data-table.presentational.html",
  styleUrls: ["./shared-data-table.presentational.scss"],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedDataTablePresentational<TResource extends BaseResource> extends SharedAbstractPresentational {
  readonly TIME_BEFORE_DISPLAY_TOOLTIP: number = environment.timeBeforeDisplayTooltipOnDataTable;

  private _datas: TResource[];
  defaultSortColumn: DataTableColumns;

  @Input()
  set datas(value: TResource[]) {
    if (this.pendingRetreiveAllForSelection) {
      this.pendingRetreiveAllForSelection = false;
      this.multiSelectionValueOnAllPages = value.map(item => item.resId);
      this.computeNumberItemSelectedOnCurrentPage();
      this.displaySelectAllElementSuccess = true;
      return;
    }
    this._datas = value;
    this.computeIsDatasPresent();
    this.computeNumberItemSelectedOnCurrentPage();
  }

  get datas(): TResource[] {
    return this._datas;
  }

  @Input()
  isMultiSelectable: boolean = false;

  @Input()
  skipInitialQuery: boolean = false;

  @Input()
  isLoading: boolean = false;

  private _queryParameters: QueryParameters;

  @Input()
  columnsSkippedToClear: string[];

  @Input()
  set queryParameters(value: QueryParameters) {
    if (isTrue(this.pendingRetreiveAllForSelection) || isTrue(this.displaySelectAllElementSuccess)) {
      return;
    }
    const newSearchItems = QueryParametersUtil.getSearchItems(value);
    const oldSearchItems = QueryParametersUtil.getSearchItems(this._queryParameters);
    if (this._isDifferentSearchItems(oldSearchItems, newSearchItems)) {
      this.multiSelectionValueOnAllPages = [];
      this.resetCounterItemSelectedOnCurrentPage();
      this.resetBannerSelectAll();
    }
    this._queryParameters = QueryParametersUtil.clone(value);
    if (isNullOrUndefined(value.paging)) {
      this._queryParameters.paging = new Paging(environment.defaultPageSize);
    }
  }

  private _isDifferentSearchItems(oldSearchItems: MappingObject, newSearchItems: MappingObject): boolean {
    return true;
  }

  get queryParameters(): QueryParameters {
    return this._queryParameters;
  }

  private _columns: DataTableColumns[];

  @Input()
  set columns(value: DataTableColumns[]) {
    this._columns = value;
    this.defaultSortColumn = this._columns.find(c => isTrue(c.isSortable) && c.order !== 0);
  }

  get columns(): DataTableColumns[] {
    return this._columns;
  }

  private _actions: DataTableActions[] = [];

  @Input()
  set actions(value: DataTableActions[]) {
    this._actions = value;
  }

  get actions(): DataTableActions[] {
    return this._actions;
  }

  private _bulkActions: DataTableBulkActions[] = [];

  @Input()
  set bulkActions(value: DataTableBulkActions[]) {
    this._bulkActions = value;
  }

  get bulkActions(): DataTableBulkActions[] {
    return this._bulkActions;
  }

  private _listHighlightingId: string[] = [];

  @Input()
  set listHighlightingId(value: string[]) {
    this._listHighlightingId = isNullOrUndefined(value) ? [] : value;
  }

  get listHighlightingId(): string[] {
    return isNullOrUndefined(this._listHighlightingId) ? [] : this._listHighlightingId;
  }

  @Input()
  scrollHeight: string | undefined = undefined;

  private _listCdkDropListId: string[];
  listCdkDropListIdPrefixed: string[];

  @Input()
  set listCdkDropListId(value: string[]) {
    this._listCdkDropListId = value;
    this.listCdkDropListIdPrefixed = value.map(v => environment.cdkDropListIdPrefix + v);
  }

  get listCdkDropListId(): string[] {
    return this._listCdkDropListId;
  }

  @Input()
  isDraggable: boolean;

  @Input()
  draggablePreviewMatIcon: string | undefined = undefined;

  @Input()
  draggablePreviewColumn: keyof TResource | string | undefined = undefined;

  private readonly _selectBS: BehaviorSubject<TResource | undefined> = new BehaviorSubject<TResource | undefined>(undefined);
  @Output("selectChange")
  readonly selectObs: Observable<TResource | undefined> = ObservableUtil.asObservable(this._selectBS);

  private readonly _multiSelectionBS: BehaviorSubject<string[]> = new BehaviorSubject<string[]>([]);
  @Output("multiSelectionChange")
  readonly multiSelectionObs: Observable<string[]> = ObservableUtil.asObservable(this._multiSelectionBS);

  set multiSelectionValueOnAllPages(value: string[]) {
    this._multiSelectionBS.next(isNullOrUndefined(value) ? [] : value);
  }

  get multiSelectionValueOnAllPages(): string[] {
    return isNullOrUndefined(this._multiSelectionBS.value) ? [] : this._multiSelectionBS.value;
  }

  multiSelectionValueOnCurrentPageCounter: number = 0;

  private readonly _queryParametersBS: BehaviorSubject<QueryParameters | undefined> = new BehaviorSubject<QueryParameters | undefined>(undefined);
  @Output("queryParametersChange")
  readonly queryParametersObs: Observable<QueryParameters | undefined> = ObservableUtil.asObservable(this._queryParametersBS);

  private readonly _queryParametersWithoutRefreshBS: BehaviorSubject<QueryParameters | undefined> = new BehaviorSubject<QueryParameters | undefined>(undefined);
  @Output("queryParametersWithoutRefreshChange")
  readonly queryParametersWithoutRefreshObs: Observable<QueryParameters | undefined> = ObservableUtil.asObservable(this._queryParametersWithoutRefreshBS);

  @ViewChild("paginator", {static: false})
  paginator: SharedPaginatorPresentational;

  isDatasPresent: boolean;
  displayPropositionSelectAllResult: boolean = false;
  pendingRetreiveAllForSelection: boolean = false;
  displaySelectAllElementSuccess: boolean = false;

  get fieldTypeEnum(): typeof FieldTypeEnum {
    return FieldTypeEnum;
  }

  get dataTableComponentEnum(): typeof DataTableComponentEnum {
    return DataTableComponentEnum;
  }

  private readonly _SEPARATOR: string = ".";

  constructor(readonly _breakpointService: BreakpointService,
              private readonly _dialog: MatDialog,
              private readonly _changeDetector: ChangeDetectorRef,
              private readonly _notificationService: NotificationService) {
    super();
  }

  private resetCounterItemSelectedOnCurrentPage(): void {
    this.multiSelectionValueOnCurrentPageCounter = 0;
  }

  private resetBannerSelectAll(): void {
    this.displayPropositionSelectAllResult = false;
    this.displaySelectAllElementSuccess = false;
  }

  private computeNumberItemSelectedOnCurrentPage(): void {
    this.resetCounterItemSelectedOnCurrentPage();
    this.multiSelectionValueOnAllPages.forEach(idItemSelected => {
      if (!isNullOrUndefined(this.datas) && !isNullOrUndefined(this.datas.find(data => data.resId === idItemSelected))) {
        this.multiSelectionValueOnCurrentPageCounter++;
      }
    });
  }

  private computeIsDatasPresent(): void {
    let isDatasPresent = false;
    if (!isNullOrUndefined(this.datas) && this.datas.length > 0) {
      isDatasPresent = true;
    }
    this.isDatasPresent = isDatasPresent;
  }

  showDetail(row: TResource): void {
    this._selectBS.next(row);
  }

  getCellData(row: TResource, column: DataTableColumns): any {
    const data = this.getData(row, column);
    if (column.translate && column.filterEnum) {
      const enumValue = column.filterEnum.find(f => f.key === data + "");
      if (enumValue) {
        return enumValue.value;
      }
      return data;
    }
    return this.applyTypeToData(data, column.type);
  }

  getData(row: TResource, column: DataTableColumns): any {
    const field = column.field;
    if (field.includes(this._SEPARATOR)) {
      return ObjectUtil.getNestedValue(row as any, column.field);
    }
    return row[field];
  }

  private applyTypeToData(data: any, type: FieldTypeEnum): any {
    if (isNullOrUndefined(data)) {
      return StringUtil.stringEmpty;
    }
    if (type === FieldTypeEnum.date) {
      return DateUtil.convertDateToDateString(data);
    }
    if (type === FieldTypeEnum.datetime) {
      return DateUtil.convertDateToDateTimeString(data);
    }
    return data;
  }

  backendSort($event: LazyLoadEvent): void {
    if (isTrue(this.skipInitialQuery)) {
      this.skipInitialQuery = false;
      return;
    }
    const queryParameters = ObjectUtil.clone(this.queryParameters);
    if (!isNullOrUndefined($event.sortField)) {
      const sort = this.adaptLazyLoadEventToSortModel($event);
      queryParameters.sort = sort;
    }
    this._queryParametersBS.next(queryParameters);
  }

  private adaptLazyLoadEventToSortModel($event: LazyLoadEvent): Sort {
    return {
      field: this.getSortableFieldByFieldName($event.sortField),
      order: $event.sortOrder === -1 ? OrderEnum.descending : OrderEnum.ascending,
    } as Sort;
  }

  pageChange($event: Paging): void {
    this.resetCounterItemSelectedOnCurrentPage();
    this.resetBannerSelectAll();
    this.queryParameters.paging = $event;
    this._queryParametersBS.next(this.queryParameters);
  }

  searchChange(col: DataTableColumns, value: string | BaseResource): void {
    if (this.multiSelectionValueOnAllPages.length > 0) {
      this.subscribe(this._dialog.open(SharedDataTableLooseSelectionDialog, {
        data: {
          numberItemSelected: this.multiSelectionValueOnAllPages.length,
        } as SharedDataTableLooseSelectionDialogData,
      }).afterClosed().pipe(
        tap((isConfirmed: boolean | undefined) => {
          if (isNullOrUndefined(isConfirmed)) {
            this._rollbackChangeOnFilter(col);
            return;
          }
          if (isTrue(isConfirmed)) {
            this._searchChange(col, value);
          }
        }),
      ));
    } else {
      this._searchChange(col, value);
    }
  }

  private _searchChange(col: DataTableColumns, value: string | BaseResource): void {
    this.multiSelectionValueOnAllPages = [];
    this.resetCounterItemSelectedOnCurrentPage();
    this.resetBannerSelectAll();

    const fieldName = this.getFilterableField(col);
    if (isEmptyString(value) || isNullOrUndefined(value)) {
      MappingObjectUtil.delete(QueryParametersUtil.getSearchItems(this.queryParameters), fieldName);
    } else {
      if (col.type === FieldTypeEnum.searchableSingleSelect) {
        value = (value as BaseResource).resId;
      }
      MappingObjectUtil.set(QueryParametersUtil.getSearchItems(this.queryParameters), fieldName, value);
    }
    const queryParameters = QueryParametersUtil.resetToFirstPage(this.queryParameters);
    this._queryParametersBS.next(queryParameters);
  }

  private _rollbackChangeOnFilter(col: DataTableColumns<TResource>): void {
    col.isFilterable = false;
    this._changeDetector.detectChanges();
    setTimeout(() => {
      col.isFilterable = true;
      this._changeDetector.detectChanges();
    }, 0);
  }

  getValue(col: DataTableColumns<TResource>): string | null {
    const value = MappingObjectUtil.get(this.queryParameters.search.searchItems, this.getFilterableField(col));
    if (value === null || value === undefined || value === StringUtil.stringEmpty) {
      return null;
    }
    return value;
  }

  private getFilterableField(col: DataTableColumns<TResource>): string {
    return isNullOrUndefined(col.filterableField) ? col.field : col.filterableField;
  }

  private getSortableField(col: DataTableColumns): string {
    return isNullOrUndefined(col.sortableField) ? col.field : col.sortableField;
  }

  getSortableFieldByFieldName(fieldName: string): string {
    const column = this.columns.find(col => col.field === fieldName);
    return this.getSortableField(column);
  }

  isFilterApplied(): boolean {
    let toSkip: boolean = false;
    if (MappingObjectUtil.size(QueryParametersUtil.getSearchItems(this.queryParameters)) > 0) {
      if (this.columnsSkippedToClear === undefined || isEmptyArray(this.columnsSkippedToClear)) {
        return true;
      }
      const searchItems: Map<string, string> = MappingObjectUtil.toMap(this.queryParameters.search.searchItems);
      searchItems.forEach((value: string, key: string) => {
        if (this.columnsSkippedToClear.some(c => c.includes(key)) === false) {
          toSkip = true;
        }
      });
    }
    return toSkip;
  }

  clearFilters(): void {
    if (this.multiSelectionValueOnAllPages.length > 0) {
      this.subscribe(this._dialog.open(SharedDataTableLooseSelectionDialog, {
        data: {
          numberItemSelected: this.multiSelectionValueOnAllPages.length,
        } as SharedDataTableLooseSelectionDialogData,
      }).afterClosed().pipe(
        tap((isConfirmed: boolean | undefined) => {
          if (isNullOrUndefined(isConfirmed)) {
            return;
          }
          if (isTrue(isConfirmed)) {
            this._clearFilters();
          }
        }),
      ));
    } else {
      this._clearFilters();
    }
  }

  private _clearFilters(): void {
    this.multiSelectionValueOnAllPages = [];
    this.resetCounterItemSelectedOnCurrentPage();
    this.resetBannerSelectAll();

    let queryParameters = QueryParametersUtil.clone(this.queryParameters);
    const searchItems = QueryParametersUtil.getSearchItems(queryParameters);
    if (this.columnsSkippedToClear === undefined || isEmptyArray(this.columnsSkippedToClear)) {
      MappingObjectUtil.clear(searchItems);
    } else {
      MappingObjectUtil.forEach(searchItems, (value: string, key: string) => {
        if (this.columnsSkippedToClear.some(c => c.includes(key)) === false) {
          MappingObjectUtil.delete(searchItems, key);
        }
      });
    }
    queryParameters = QueryParametersUtil.resetToFirstPage(queryParameters);
    this._queryParametersBS.next(queryParameters);
  }

  callCallback($event: MouseEvent, callback: (current: TResource) => void, rowData: TResource): void {
    this.avoidOpen($event);
    callback(rowData);
  }

  isHighlighting(rowData: TResource): boolean {
    if (isEmptyArray(this.listHighlightingId) && isEmptyArray(this.multiSelectionValueOnAllPages)) {
      return false;
    }
    if (this.listHighlightingId.includes(rowData.resId)) {
      return true;
    }
    return this.multiSelectionValueOnAllPages.includes(rowData.resId);
  }

  displayActionOnCondition(action: DataTableActions<TResource>, resource: TResource = undefined): boolean | Observable<boolean> {
    if (isNullOrUndefined(resource) || isNullOrUndefined(action.displayOnCondition)) {
      return this.atLeastOneActionRowDisplayed(action);
    }
    return action.displayOnCondition(resource);
  }

  private atLeastOneActionRowDisplayed(action: DataTableActions<TResource>): boolean | Observable<boolean> {
    if (isNullOrUndefined(this._datas) || isEmptyArray(this._datas)) {
      return false;
    }
    if (isNullOrUndefined(action.displayOnCondition)) {
      return true;
    }
    if (this._datas.some(data => !isObservable(action.displayOnCondition(data)) && isTrue(action.displayOnCondition(data)))) {
      return true;
    }
    // TODO Find a way to merge all observable
    return action.displayOnCondition(this._datas[0]);
  }

  getWidth(column: DataTableColumns<TResource>): string {
    if (!isNullOrUndefined(column.width)) {
      return column.width;
    }
    if (column.component === DataTableComponentEnum.conformityLevelStar) {
      return "100px";
    }
    if (column.type === FieldTypeEnum.datetime) {
      return "145px";
    }
    if (column.type === FieldTypeEnum.date) {
      return "90px";
    }

    return undefined;
  }

  isColumnFilterable(): boolean {
    return !isNullOrUndefined(this.columns.find(c => isTrue(c.isFilterable)));
  }

  toggleCheckbox($event: MatCheckboxChange, rowData: TResource): void {
    const listRowSelected = this._multiSelectionBS.value;
    if ($event.checked) {
      listRowSelected.push(rowData.resId);
      this.multiSelectionValueOnCurrentPageCounter++;
    } else {
      const index = listRowSelected.indexOf(rowData.resId);
      if (index === -1) {
        return;
      }
      this.multiSelectionValueOnCurrentPageCounter--;
      listRowSelected.splice(index, 1);
    }
    this._multiSelectionBS.next(listRowSelected);
  }

  avoidOpen($event: MouseEvent): void {
    $event.stopPropagation();
  }

  toggleCurrentPageCheckbox($event: MatCheckboxChange): void {
    const listRowSelectedOnAllPages = this.multiSelectionValueOnAllPages;
    this.datas.forEach((item: TResource) => {
      const indexOf = listRowSelectedOnAllPages.indexOf(item.resId);
      if ($event.checked) {
        this.displayPropositionSelectAllResult = true;
        if (indexOf !== -1) {
          return;
        }
        listRowSelectedOnAllPages.push(item.resId);
      } else {
        this.resetBannerSelectAll();
        if (indexOf === -1) {
          return;
        }
        listRowSelectedOnAllPages.splice(indexOf, 1);
      }
    });
    this._multiSelectionBS.next(listRowSelectedOnAllPages);
    this.computeNumberItemSelectedOnCurrentPage();
    if ($event.checked && this.datas.length === this.queryParameters.paging.length) {
      this.resetBannerSelectAll();
    }
  }

  isLineSelected(rowData: TResource): boolean {
    return this.multiSelectionValueOnAllPages.includes(rowData.resId);
  }

  selectAllResult(): Observable<string[]> {
    if (this.queryParameters.paging.length > environment.maximalPageSizeToRetrievePaginationInfo) {
      this._notificationService.showInformation(TRANSLATE("table.notification.onlyTheMaximalAmountAreSelected"), {maximalNumber: environment.maximalPageSizeToRetrievePaginationInfo} as any);
    }
    this.pendingRetreiveAllForSelection = true;
    const queryParameters = QueryParametersUtil.clone(this.queryParameters);
    queryParameters.paging = new Paging(environment.maximalPageSizeToRetrievePaginationInfo);
    this._queryParametersBS.next(queryParameters);
    this._queryParametersWithoutRefreshBS.next(QueryParametersUtil.clone(this.queryParameters));
    return this.multiSelectionObs;
  }

  cleanSelection(): void {
    this.resetBannerSelectAll();
    this.multiSelectionValueOnAllPages = [];
    this.resetCounterItemSelectedOnCurrentPage();
  }

  callBulkAction(bulkAction: DataTableBulkActions): void {
    bulkAction.callback(this.multiSelectionValueOnAllPages);
    this.cleanSelection();
  }

  dragStart($event: CdkDragStart): void {
    this.cleanSelection();
  }

  cleanFilter(select: MatSelect | HTMLInputElement, col: DataTableColumns<TResource>): void {
    select.value = StringUtil.stringEmpty;
    this.searchChange(col, StringUtil.stringEmpty);
  }

  isDisabledAction(action: DataTableActions<TResource>, rowData: TResource): boolean | Observable<boolean> {
    if (isNullOrUndefined(action.disableCondition)) {
      return false;
    }
    return action.disableCondition(rowData);
  }
}



