import {
  ChangeDetectionStrategy,
  Component,
  HostBinding,
  HostListener,
  Input,
} from "@angular/core";
import {MatDialog} from "@angular/material/dialog";
import {
  SharedInfoExcludedIgnoredFileDialog,
  SharedInfoExcludedIgnoredFileDialogData,
} from "@shared/components/dialogs/shared-info-excluded-ignored-file/shared-info-excluded-ignored-file.dialog";
import {SharedAbstractPresentational} from "@shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {DataFileStatusEnum} from "@shared/enums/business/data-file-status.enum";
import {TRANSLATE} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-datafile-quick-status",
  templateUrl: "./shared-datafile-quick-status.presentational.html",
  styleUrls: ["./shared-datafile-quick-status.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})

export class SharedDatafileQuickStatusPresentational extends SharedAbstractPresentational {
  private readonly CLASS_READY: string = "ready";
  private readonly CLASS_ERROR: string = "error";
  private readonly CLASS_WARNING: string = "warning";
  private readonly CLASS_CLEANED: string = "cleaned";
  private readonly CLASS_CAN_SEE_DETAIL: string = "can-see-detail";

  readonly CLASS_PENDING: string = "pending";

  _value: DataFileStatusEnum;

  @Input()
  set value(value: DataFileStatusEnum) {
    this._value = value;
    this.classes = this.getColor();
  }

  get value(): DataFileStatusEnum {
    return this._value;
  }

  constructor(private readonly _dialog: MatDialog) {
    super();
  }

  toolipToTranslate: string | undefined;

  @HostBinding("class")
  classes: string;

  @HostListener("click", ["$event"]) click(mouseEvent: MouseEvent): void {
    mouseEvent.stopPropagation();
    if (this.value !== DataFileStatusEnum.EXCLUDED_FILE && this.value !== DataFileStatusEnum.IGNORED_FILE) {
      return;
    }
    this._dialog.open(SharedInfoExcludedIgnoredFileDialog, {
      data: {
        dataFileStatusEnum: this.value,
      } as SharedInfoExcludedIgnoredFileDialogData,
    });
  }

  getColor(): string {
    switch (this.value) {
      case DataFileStatusEnum.READY:
        this.toolipToTranslate = TRANSLATE("dataFileQuickStatus.ready");
        return this.CLASS_READY;
      case DataFileStatusEnum.IN_ERROR:
        this.toolipToTranslate = TRANSLATE("dataFileQuickStatus.inError");
        return this.CLASS_ERROR;
      case DataFileStatusEnum.EXCLUDED_FILE:
        this.toolipToTranslate = TRANSLATE("dataFileQuickStatus.excludedFile");
        return this.CLASS_ERROR + " " + this.CLASS_CAN_SEE_DETAIL;
      case DataFileStatusEnum.IGNORED_FILE:
        this.toolipToTranslate = TRANSLATE("dataFileQuickStatus.ignoredFile");
        return this.CLASS_WARNING + " " + this.CLASS_CAN_SEE_DETAIL;
      case DataFileStatusEnum.CLEANED:
        this.toolipToTranslate = TRANSLATE("dataFileQuickStatus.cleaned");
        return this.CLASS_CLEANED;
      default:
        this.toolipToTranslate = TRANSLATE("dataFileQuickStatus.pending");
        return this.CLASS_PENDING;
    }
  }
}
