import {DOCUMENT} from "@angular/common";
import {
  AfterViewInit,
  ChangeDetectionStrategy,
  Component,
  Inject,
  Input,
  Renderer2,
  ViewChild,
} from "@angular/core";
import {
  MatMenu,
  MatMenuItem,
} from "@angular/material/menu";
import {SharedAbstractPresentational} from "@shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {BaseResourceApiEnum} from "@shared/enums/api.enum";
import {urlSeparator} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-toc",
  templateUrl: "./shared-toc.presentational.html",
  styleUrls: ["./shared-toc.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedTocPresentational extends SharedAbstractPresentational implements AfterViewInit {
  @Input()
  quickStart: string;

  @Input()
  integrationGuide: string;

  @Input()
  apis: string;

  @Input()
  toolsGuide: string;

  @Input()
  isLoading: boolean;

  @ViewChild("menu", {static: false})
  readonly menu: MatMenu;

  @ViewChild("integrationGuideMenu", {static: false})
  readonly integrationGuideMenu: MatMenu;

  matMenuLinkUpdated: MatMenuItem[] = [];
  readonly CLASS_MAT_MENU_WITH_LINK: string = "shared-toc-container-panel";

  constructor(@Inject(DOCUMENT) private readonly _document: Document,
              private readonly _renderer: Renderer2) {
    super();
  }

  ngAfterViewInit(): void {
    super.ngAfterViewInit();

    this.subscribe(this.menu._hovered(), (matMenuItem: MatMenuItem) => {
      if (this.matMenuLinkUpdated.includes(matMenuItem)) {
        return;
      }
      this.matMenuLinkUpdated.push(matMenuItem);
      setTimeout(() => {
        const listA = this._document.querySelectorAll(`.${this.CLASS_MAT_MENU_WITH_LINK} a:not([active="true"])`);
        listA.forEach(a => {
          const existingHref = a.attributes["href"].value;
          this._renderer.setAttribute(a, "href", BaseResourceApiEnum.docs + urlSeparator + existingHref);
          this._renderer.setAttribute(a, "active", "true");
        });
      }, 0);
    });
  }
}
