import {
  ChangeDetectorRef,
  Component,
  Input,
  OnInit,
  Output,
} from "@angular/core";
import {
  AbstractControl,
  ControlValueAccessor,
  FormControl,
  NG_VALUE_ACCESSOR,
} from "@angular/forms";
import {SharedAbstractPresentational} from "@shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {BaseFormDefinition} from "@shared/models/base-form-definition.model";
import {
  BehaviorSubject,
  Observable,
} from "rxjs";
import {
  distinctUntilChanged,
  pairwise,
  tap,
} from "rxjs/operators";
import {
  ObservableUtil,
  PropertyName,
  TRANSLATE,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-url-input-navigator",
  templateUrl: "./shared-url-input-navigation.presentational.html",
  styleUrls: ["./shared-url-input-navigation.presentational.scss"],
  providers: [
    {
      provide: NG_VALUE_ACCESSOR,
      multi: true,
      useExisting: SharedUrlInputNavigationPresentational,
    },
  ],
})

export class SharedUrlInputNavigationPresentational extends SharedAbstractPresentational implements OnInit, ControlValueAccessor {
  formDefinition: FormComponentFormDefinition = new FormComponentFormDefinition();

  @Input()
  placeholderToTranslate: string = TRANSLATE("shared.UrlInput.placeholder");

  @Input()
  formControl: FormControl;

  @Input()
  readonly: boolean;

  private readonly _valueBS: BehaviorSubject<string[] | undefined> = new BehaviorSubject<string[] | undefined>(undefined);
  @Output("valueChange")
  readonly valueObs: Observable<string[] | undefined> = ObservableUtil.asObservable(this._valueBS);

  constructor(private readonly _changeDetector: ChangeDetectorRef) {
    super();
  }

  ngOnInit(): void {
    this.subscribe(this.formControl.valueChanges.pipe(
      distinctUntilChanged(),
      pairwise(),
      tap(([prev, next]) => {
        this.propagateChange(next);
        this._valueBS.next(next);
        if (prev === next) {
          this.formControl.markAsPristine();
        }
        this._changeDetector.detectChanges();
      }),
    ));
  }

  propagateChange = (__: any) => {};

  registerOnChange(fn: any): void {
    this.propagateChange = fn;
  }

  registerOnTouched(fn: any): void {
  }

  setDisabledState(isDisabled: boolean): void {
  }

  writeValue(value: string[]): void {
  }

  navigateToUrl(fd: AbstractControl): void {
    window.open(fd.value, "_blank");
  }

}

class FormComponentFormDefinition extends BaseFormDefinition {
  @PropertyName() url: string;
}

