export const urlSeparator: string = "/";

export enum AppRoutesEnum {
  root = "",
  index = "index",
  login = "login",
  homePage = "home",
  deposit = "deposit",
  aip = "aip",
  organizationalUnit = "organizational-unit",
  admin = "admin",
  preservation = "preservation",
  order = "order",
  separator = "/",
  paramId = ":id",
  paramIdOrgUnit = ":idOrgUnit",
  paramIdWithoutPrefixParam = "id",
  paramIdOrgUnitWithoutPrefixParam = "idOrgUnit",
  paramIdExecution = ":idExecution",
  paramIdExecutionWithoutPrefixParam = "idExecution",
  edit = "edit",
}

export enum HomePageRoutesEnum {
  detail = "detail",
  browsing = "browsing",
}

export enum DepositRoutesEnum {
  create = "create",
  detail = "detail",
  edit = "edit",
  metadata = "metadata",
  files = "files",
  upload = "upload",
  collection = "collection",
  paramTab = ":tab",
  paramTabWithoutPrefix = "tab",
}

export enum OrganizationalUnitRoutesEnum {
  detail = "detail",
  edit = "edit",
}

export enum AdminRoutesEnum {
  submissionPolicy = "submission-policy",
  submissionPolicyCreate = "create",
  submissionPolicyDetail = "detail",
  submissionPolicyEdit = "edit",

  preservationPolicy = "preservation-policy",
  preservationPolicyDetail = "detail",
  preservationPolicyCreate = "create",
  preservationPolicyEdit = "edit",

  disseminationPolicy = "dissemination-policy",
  disseminationPolicyDetail = "detail",
  disseminationPolicyCreate = "create",
  disseminationPolicyEdit = "edit",

  license = "license",
  licenseCreate = "create",
  licenseEdit = "edit",
  licenseDetail = "detail",

  organizationalUnit = "organizational-unit",
  organizationalUnitCreate = "create",
  organizationalUnitDetail = "detail",
  organizationalUnitEdit = "edit",

  institution = "institution",
  institutionCreate = "create",
  institutionDetail = "detail",
  institutionEdit = "edit",

  researchDomain = "research-domain",

  language = "language",

  user = "user",
  userDetail = "detail",
  userCreate = "create",
  userEdit = "edit",

  oaiSet = "oai-set",
  oaiSetDetail = "detail",
  oaiSetCreate = "create",
  oaiSetBulkCreate = "bulk-create",
  oaiSetEdit = "edit",

  oauth2client = "oauth2-clients",
  oauth2clientCreate = "create",
  oauth2clientDetail = "detail",
  oauth2clientEdit = "edit",

  person = "person",
  personDetail = "detail",
  personCreate = "create",
  personEdit = "edit",

  role = "role",
  roleDetail = "detail",
  roleCreate = "create",
  roleEdit = "edit",

  fundingAgencies = "funding-agencies",
  fundingAgenciesDetail = "detail",
  fundingAgenciesCreate = "create",
  fundingAgenciesEdit = "edit",

  metadataType = "metadata-type",
  metadataTypeCreate = "create",
  metadataTypeEdit = "edit",
  metadataTypeDetail = "detail",
}

export enum OrderRoutesEnum {
  order = "order",
  allOrder = "all-order",
  allOrderCreate = "create",
  allOrderDetail = "detail",
  allOrderEdit = "edit",
  myOrder = "my-order",
  myOrderCreate = "create",
  myOrderDetail = "detail",
  myOrderEdit = "edit",
}

export enum PreservationPlanningRoutesEnum {
  preservationPlanning = "preservation-planning",
  preservationPlanningCreate = "create",
  preservationPlanningDetail = "detail",
  preservationPlanningEdit = "edit",
  preservationPlanningExecution = "execution",

  monitoring = "monitoring",

  aipStatuses = "aip-statuses",

  job = "job",
  jobCreate = "create",
  jobDetail = "detail",
  jobEdit = "edit",
  execution = "execution",

  aip = "aip",
  aipCreate = "create",
  aipDetail = "detail",
  aipList = "list",
  aipTabAll = "all",
  aipTabUnit = "unit",
  aipTabCollections = "collection",
  aipMetadata = "metadata",
  aipFiles = "files",
  aipNotFound = "not-found",

  aipDownloaded = "aip-downloaded",
  aipDownloadedCreate = "create",
  aipDownloadedDetail = "detail",
  aipDownloadedTabAll = "all",
  aipDownloadedTabUnit = "unit",
  aipDownloadedTabCollections = "collection",
  aipDownloadedMetadata = "metadata",
  aipDownloadedFiles = "files",
  aipDownloadedNotFound = "not-found",

  storagionNumber = ":storagion",
  storagionNumberWithoutPrefixParam = "storagion",

  tab = ":tab",
  tabWithoutPrefixParam = "tab",

  dip = "dip",
  dipDetail = "detail",
  dipEdit = "edit",
  dipMetadata = "metadata",
  dipFiles = "files",

  sip = "sip",
  sipDetail = "detail",
  sipMetadata = "metadata",
  sipFiles = "files",
}

export class RoutesEnum implements RoutesEnum {
  static index: string = AppRoutesEnum.index;
  static login: string = AppRoutesEnum.login;

  static homePage: string = AppRoutesEnum.homePage;
  static homePageDetail: string = AppRoutesEnum.homePage + urlSeparator + HomePageRoutesEnum.detail;
  static homePageBrowsing: string = AppRoutesEnum.homePage + urlSeparator + HomePageRoutesEnum.browsing;

  static deposit: string = AppRoutesEnum.deposit;
  static depositCreate: string = AppRoutesEnum.deposit + urlSeparator + DepositRoutesEnum.create;
  static depositDetail: string = AppRoutesEnum.deposit + urlSeparator + DepositRoutesEnum.detail;

  static organizationalUnit: string = AppRoutesEnum.organizationalUnit;
  static organizationalUnitDetail: string = AppRoutesEnum.organizationalUnit + urlSeparator + OrganizationalUnitRoutesEnum.detail;

  static order: string = AppRoutesEnum.order;

  static orderAllOrder: string = AppRoutesEnum.order + urlSeparator + OrderRoutesEnum.allOrder;
  static orderAllOrderDetail: string = AppRoutesEnum.order + urlSeparator + OrderRoutesEnum.allOrder + urlSeparator + OrderRoutesEnum.allOrderDetail;
  static orderAllOrderCreate: string = AppRoutesEnum.order + urlSeparator + OrderRoutesEnum.allOrder + urlSeparator + OrderRoutesEnum.allOrderCreate;

  static orderMyOrder: string = AppRoutesEnum.order + urlSeparator + OrderRoutesEnum.myOrder;
  static orderMyOrderDetail: string = AppRoutesEnum.order + urlSeparator + OrderRoutesEnum.myOrder + urlSeparator + OrderRoutesEnum.myOrderDetail;

  static admin: string = AppRoutesEnum.admin;

  static adminSubmissionPolicy: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.submissionPolicy;
  static adminSubmissionPolicyCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.submissionPolicy + urlSeparator + AdminRoutesEnum.submissionPolicyCreate;
  static adminSubmissionPolicyDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.submissionPolicy + urlSeparator + AdminRoutesEnum.submissionPolicyDetail;

  static adminPreservationPolicy: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.preservationPolicy;
  static adminPreservationPolicyCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.preservationPolicy + urlSeparator + AdminRoutesEnum.preservationPolicyCreate;
  static adminPreservationPolicyDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.preservationPolicy + urlSeparator + AdminRoutesEnum.preservationPolicyDetail;

  static adminDisseminationPolicy: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.disseminationPolicy;
  static adminDisseminationPolicyCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.disseminationPolicy + urlSeparator + AdminRoutesEnum.disseminationPolicyCreate;
  static adminDisseminationPolicyDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.disseminationPolicy + urlSeparator + AdminRoutesEnum.disseminationPolicyDetail;

  static adminLicense: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.license;
  static adminLicenseCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.license + urlSeparator + AdminRoutesEnum.licenseCreate;
  static adminLicenseDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.license + urlSeparator + AdminRoutesEnum.licenseDetail;

  static adminOrganizationalUnit: string = AppRoutesEnum.admin + urlSeparator + AppRoutesEnum.organizationalUnit;
  static adminOrganizationalUnitCreate: string = AppRoutesEnum.admin + urlSeparator + AppRoutesEnum.organizationalUnit + urlSeparator + AdminRoutesEnum.organizationalUnitCreate;
  static adminOrganizationalUnitDetail: string = AppRoutesEnum.admin + urlSeparator + AppRoutesEnum.organizationalUnit + urlSeparator + AdminRoutesEnum.organizationalUnitDetail;

  static adminInstitution: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.institution;
  static adminInstitutionCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.institution + urlSeparator + AdminRoutesEnum.institutionCreate;
  static adminInstitutionDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.institution + urlSeparator + AdminRoutesEnum.institutionDetail;

  static adminResearchDomain: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.researchDomain;

  static adminLanguage: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.language;

  static adminUser: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.user;
  static adminUserCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.user + urlSeparator + AdminRoutesEnum.userCreate;
  static adminUserDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.user + urlSeparator + AdminRoutesEnum.userDetail;

  static adminOaiSet: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.oaiSet;
  static adminOaiSetCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.oaiSet + urlSeparator + AdminRoutesEnum.oaiSetCreate;
  static adminOaiSetBulkCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.oaiSet + urlSeparator + AdminRoutesEnum.oaiSetBulkCreate;
  static adminOaiSetDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.oaiSet + urlSeparator + AdminRoutesEnum.oaiSetDetail;

  static adminOAuth2client: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.oauth2client;
  static adminOAuth2clientCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.oauth2client + urlSeparator + AdminRoutesEnum.oauth2clientCreate;
  static adminOAuth2clientDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.oauth2client + urlSeparator + AdminRoutesEnum.oauth2clientDetail;

  static adminPerson: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.person;
  static adminPersonCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.person + urlSeparator + AdminRoutesEnum.personCreate;
  static adminPersonDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.person + urlSeparator + AdminRoutesEnum.personDetail;

  static adminRole: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.role;
  static adminRoleCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.role + urlSeparator + AdminRoutesEnum.roleCreate;
  static adminRoleDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.role + urlSeparator + AdminRoutesEnum.roleDetail;

  static adminFundingAgencies: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.fundingAgencies;
  static adminFundingAgenciesCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.fundingAgencies + urlSeparator + AdminRoutesEnum.fundingAgenciesCreate;
  static adminFundingAgenciesDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.fundingAgencies + urlSeparator + AdminRoutesEnum.fundingAgenciesDetail;

  static adminMetadataType: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.metadataType;
  static adminMetadataTypeCreate: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.metadataType + urlSeparator + AdminRoutesEnum.metadataTypeCreate;
  static adminMetadataTypeDetail: string = AppRoutesEnum.admin + urlSeparator + AdminRoutesEnum.metadataType + urlSeparator + AdminRoutesEnum.metadataTypeDetail;

  static preservation: string = AppRoutesEnum.preservation;

  static preservationAip: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aip;

  static preservationAipDownloaded: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aipDownloaded;
  static preservationAipDownloadedDetail: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aipDownloaded + urlSeparator + PreservationPlanningRoutesEnum.aipDownloadedDetail;

  static preservationSip: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.sip;
  static preservationSipDetail: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.sip + urlSeparator + PreservationPlanningRoutesEnum.sipDetail;

  static preservationDip: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.dip;
  static preservationDipDetail: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.dip + urlSeparator + PreservationPlanningRoutesEnum.dipDetail;

  static preservationPreservationPlanning: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.preservationPlanning;
  static preservationPreservationPlanningCreate: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.preservationPlanning + urlSeparator + PreservationPlanningRoutesEnum.preservationPlanningCreate;
  static preservationPreservationPlanningDetail: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.preservationPlanning + urlSeparator + PreservationPlanningRoutesEnum.preservationPlanningDetail;
  static preservationPreservationPlanningEdit: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.preservationPlanning + urlSeparator + PreservationPlanningRoutesEnum.preservationPlanningEdit;

  static preservationMonitoring: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.monitoring;

  static preservationJob: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.job;
  static preservationJobCreate: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.job + urlSeparator + PreservationPlanningRoutesEnum.jobCreate;
  static preservationJobDetail: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.job + urlSeparator + PreservationPlanningRoutesEnum.jobDetail;

  static preservationAipStatuses: string = AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aipStatuses;
}
