import {Injectable} from "@angular/core";
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
} from "@angular/router";
import {DepositAction} from "@app/features/deposit/stores/deposit.action";
import {Deposit} from "@models";
import {DepositState} from "@deposit/stores/deposit.state";
import {Store} from "@ngxs/store";
import {
  AppRoutesEnum,
  DepositRoutesEnum,
} from "@shared/enums/routes.enum";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {
  distinctUntilChanged,
  filter,
  map,
} from "rxjs/operators";
import {
  ApiService,
  isNullOrUndefined,
  ResourceState,
  RoutingUtil,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class DepositDetailTabGuardService implements CanActivate {
  constructor(public router: Router,
              private readonly _store: Store,
              public apiService: ApiService,
              private readonly _securityService: SecurityService) {
  }

  canActivate(route: ActivatedRouteSnapshot): Observable<boolean> | boolean {
    const path = route.url[0].path;
    let depositId: string = route.parent.parent.params[AppRoutesEnum.paramIdWithoutPrefixParam];
    if (isNullOrUndefined(depositId)) {
      depositId = route.parent.params[AppRoutesEnum.paramIdWithoutPrefixParam];
    }
    // const depositInState = ResourceState.currentSnapshot(this._store, DepositState);
    // if (!isNullOrUndefined(depositInState) && depositId === depositInState.resId) {
    //   return this._canGoToThisTabRoute(depositInState, path, route);
    // }
    this._store.dispatch(new DepositAction.GetById(depositId, true));
    return ResourceState.current(this._store, DepositState).pipe(
      distinctUntilChanged(),
      filter(deposit => !isNullOrUndefined(deposit) && deposit.resId === depositId),
      map(deposit => this._canGoToThisTabRoute(deposit, path, route)),
    );
  }

  private _canGoToThisTabRoute(deposit: Deposit, path: string, route: ActivatedRouteSnapshot): boolean {
    if (deposit.dataFileNumber > 0 && deposit.collectionSize > 0) {
      return true;
    }
    if (path === DepositRoutesEnum.files && deposit.collectionSize > 0) {
      const url = RoutingUtil.rebuildUrl(route.pathFromRoot);
      url[url.length - 1] = DepositRoutesEnum.collection;
      this.router.navigate(url);
      return false;
    }
    if (path === DepositRoutesEnum.collection && deposit.dataFileNumber > 0) {
      const url = RoutingUtil.rebuildUrl(route.pathFromRoot);
      url[url.length - 1] = DepositRoutesEnum.files;
      this.router.navigate(url);
      return false;
    }
    return true;
  }
}
