import {Injectable} from "@angular/core";
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
} from "@angular/router";
import {
  Actions,
  Store,
} from "@ngxs/store";
import {OrgUnitAction} from "@orgunit/stores/organizational-unit.action";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {map} from "rxjs/operators";
import {
  ApiService,
  NotificationService,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class OrganizationalUnitComputeIsManagerGuardService implements CanActivate {
  constructor(public router: Router,
              public store: Store,
              public apiService: ApiService,
              private readonly _securityService: SecurityService,
              private readonly _notificationService: NotificationService,
              private readonly _actions$: Actions) {
  }

  canActivate(route: ActivatedRouteSnapshot): Observable<boolean> {
    return this._securityService.isManagerOfOrgUnit(route.params[AppRoutesEnum.paramIdWithoutPrefixParam]).pipe(
      map(authorized => {
        this.store.dispatch(new OrgUnitAction.SaveCurrentUserIsManager(authorized));
        return true;
      }));
  }
}
