import {Injectable} from "@angular/core";
import {
  ActivatedRouteSnapshot,
  CanActivate,
  Router,
} from "@angular/router";
import {Store} from "@ngxs/store";
import {AppRoutesEnum} from "@shared/enums/routes.enum";
import {SecurityService} from "@shared/services/security.service";
import {Observable} from "rxjs";
import {tap} from "rxjs/operators";
import {
  ApiService,
  NotificationService,
  TRANSLATE,
} from "solidify-frontend";

@Injectable({
  providedIn: "root",
})
export class OrganizationalUnitRoleGuardService implements CanActivate {
  constructor(public router: Router,
              public store: Store,
              public apiService: ApiService,
              private readonly _securityService: SecurityService,
              private readonly _notificationService: NotificationService) {
  }

  canActivate(route: ActivatedRouteSnapshot): Observable<boolean> {
    return this._securityService.isManagerOfOrgUnit(route.parent.params[AppRoutesEnum.paramIdWithoutPrefixParam]).pipe(
      tap(authorized => {
        if (authorized === false) {
          this._notificationService.showWarning(TRANSLATE("organizationalUnit.security.notification.noRightForThisAction"));
        }
      }));
  }
}
