import {
  Actions,
  ofActionCompleted,
  Store,
} from "@ngxs/store";
import {FormErrorEnum} from "@shared/enums/form-error.enum";
import {FormControlKey} from "@shared/models/form-control-key.model";
import {Observable} from "rxjs";
import {
  map,
  take,
} from "rxjs/operators";
import {
  BaseResource,
  isNullOrUndefined,
  MappingObjectUtil,
  QueryParameters,
  ResourceActionHelper,
  ResourceNameSpace,
  StringUtil,
} from "solidify-frontend";

export class CrudHelper {
  static checkAvailable(formControlKey: FormControlKey, store: Store, actions$: Actions, resourceNameSpace: ResourceNameSpace, resId: string | undefined): Observable<boolean> {
    const queryParameters = new QueryParameters();
    MappingObjectUtil.set(queryParameters.search.searchItems, formControlKey.key, formControlKey.formControl.value);
    store.dispatch(ResourceActionHelper.getAll(resourceNameSpace, queryParameters));
    return actions$.pipe(ofActionCompleted(resourceNameSpace.GetAllSuccess))
      .pipe(
        take(1),
        map(result => {
          if (result.result.successful && result.action.list._data.length > 0) {
            if (result.action.list._data.filter((f: BaseResource) =>
              (StringUtil.stringEmpty + f[formControlKey.key]).toLowerCase() === (StringUtil.stringEmpty + formControlKey.formControl.value).toLowerCase()
              && (isNullOrUndefined(resId) || f.resId !== resId)).length === 0) {
              return true;
            }
            formControlKey.formControl.setErrors({
              errorsFromBackend: [FormErrorEnum.ALREADY_USED],
            });
            formControlKey.formControl.markAsTouched();
            formControlKey.changeDetector.detectChanges();
            return false;
          }
        }),
      );
  }
}
