import {isNullOrUndefined} from "solidify-frontend";

export class SessionStorageHelper {
  private static readonly LIST_SEPARATOR: string = ";";

  static getListItem(key: string): string[] {
    let list = [];
    const listSerialized = sessionStorage.getItem(key);
    if (!isNullOrUndefined(listSerialized)) {
      list = listSerialized.split(this.LIST_SEPARATOR);
    }
    return list;
  }

  static addItemInList(key: string, item: string): string[] {
    let list = [];
    const listSerialized = sessionStorage.getItem(key);
    if (!isNullOrUndefined(listSerialized)) {
      list = listSerialized.split(this.LIST_SEPARATOR);
    }
    if (!list.includes(item)) {
      list.push(item);
      sessionStorage.setItem(key, list.join(this.LIST_SEPARATOR));
    }
    return list;
  }

  static removeItemInList(key: string, item: string): string[] {
    let list = [];
    const listSerialized = sessionStorage.getItem(key);
    if (!isNullOrUndefined(listSerialized)) {
      list = listSerialized.split(this.LIST_SEPARATOR);
    }
    const indexOfItem = list.indexOf(item);
    if (indexOfItem !== -1) {
      list.splice(indexOfItem, 1);
      if (list.length === 0) {
        sessionStorage.removeItem(key);
      } else {
        sessionStorage.setItem(key, list.join(this.LIST_SEPARATOR));
      }
    }
    return list;
  }
}
