import {MatButtonThemeClassName} from "@shared/directives/shared-mat-button-theme/shared-mat-button-theme.directive";
import {RoutesEnum} from "@shared/enums/routes.enum";
import {
  EMPTY,
  Observable,
  of,
} from "rxjs";
import {fromPromise} from "rxjs/internal-compatibility";
import {
  BaseResource,
  isNull,
  isObservable,
  isPromise,
  isUndefined,
} from "solidify-frontend";

export interface ExtraButtonToolbar<TResourceModel extends BaseResource> {
  color: "accent" | "primary" | "warn" | "success";
  class?: string | undefined;
  icon: string;
  typeButton?: MatButtonThemeClassName;
  labelToTranslate: string;
  tooltipToTranslate?: string | undefined;
  route?: RoutesEnum;
  callback?: (current?: TResourceModel | undefined) => void;
  order?: number;
  displayCondition?: (current: TResourceModel) => ObservableOrPromiseOrValue<boolean>;
  disableCondition?: (current: TResourceModel) => ObservableOrPromiseOrValue<boolean>;
}

export type ObservableOrPromiseOrValue<T> = Observable<T> | Promise<T> | T;

export function observablify<T>(source: ObservableOrPromiseOrValue<T>, emptyOnUndefined?: boolean, emptyOnNull?: boolean): Observable<T> {
  return isObservable(source) ? source
    : isPromise(source) ? fromPromise(source)
      : emptyOnUndefined && isUndefined(source) ? EMPTY
        : emptyOnNull && isNull(source) ? EMPTY
          : of(source);
}
