import {Role} from "@models";
import {AdminResourceApiEnum} from "@app/shared/enums/api.enum";
import {LocalStateEnum} from "@app/shared/enums/local-state.enum";
import {
  Actions,
  State,
  Store,
} from "@ngxs/store";
import {sharedRoleActionNameSpace} from "@shared/stores/role/shared-role.action";
import {
  ApiService,
  defaultResourceStateInitValue,
  NotificationService,
  QueryParameters,
  ResourceState,
  ResourceStateModel,
} from "solidify-frontend";
import {environment} from "../../../../environments/environment";

export interface SharedRoleStateModel extends ResourceStateModel<Role> {
}

@State<SharedRoleStateModel>({
  name: LocalStateEnum.shared_role,
  defaults: {
    ...defaultResourceStateInitValue(),
    queryParameters: new QueryParameters(environment.defaultEnumValuePageSizeOption),
  },
})
export class SharedRoleState extends ResourceState<SharedRoleStateModel, Role> {
  constructor(protected apiService: ApiService,
              protected store: Store,
              protected notificationService: NotificationService,
              protected actions$: Actions) {
    super(apiService, store, notificationService, actions$, {
      nameSpace: sharedRoleActionNameSpace,
    });
  }

  protected get _urlResource(): string {
    return AdminResourceApiEnum.roles;
  }
}
