import {ApplicationRoleEnum} from "@app/shared/enums/application-role.enum";
import {PermissionUtil} from "@app/shared/utils/permission.util";

describe("RoleUtil", () => {
  describe("isRootPermission", () => {
    it("should return true when user is ROOT", () => {
      const userRoles = [ApplicationRoleEnum.root];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeTruthy();
    });

    it("should return false when user is ADMIN", () => {
      const userRoles = [ApplicationRoleEnum.admin];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeFalsy();
    });

    it("should return false when user is USER", () => {
      const userRoles = [ApplicationRoleEnum.user];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeFalsy();
    });

    it("should return false when user is GUEST", () => {
      const userRoles = [ApplicationRoleEnum.guest];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeFalsy();
    });

    it("should return false when user have no role", () => {
      const userRoles = [];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeFalsy();
    });

    it("should return true when user is [ROOT, ADMIN]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.user];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeTruthy();
    });

    it("should return false when user is [ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeFalsy();
    });

    it("should return false when user is [GUEST, USER]", () => {
      const userRoles = [ApplicationRoleEnum.guest, ApplicationRoleEnum.user];
      expect(PermissionUtil.isRootPermission(userRoles)).toBeFalsy();
    });
  });

  describe("isAdminPermission", () => {
    it("should return true when user is ROOT", () => {
      const userRoles = [ApplicationRoleEnum.root];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is ADMIN", () => {
      const userRoles = [ApplicationRoleEnum.admin];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeTruthy();
    });

    it("should return false when user is USER", () => {
      const userRoles = [ApplicationRoleEnum.user];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeFalsy();
    });

    it("should return false when user is GUEST", () => {
      const userRoles = [ApplicationRoleEnum.guest];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeFalsy();
    });

    it("should return false when user have no role", () => {
      const userRoles = [];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeFalsy();
    });

    it("should return true when user is [ROOT, ADMIN]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.user];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeTruthy();
    });

    it("should return false when user is [GUEST, USER]", () => {
      const userRoles = [ApplicationRoleEnum.guest, ApplicationRoleEnum.user];
      expect(PermissionUtil.isAdminPermission(userRoles)).toBeFalsy();
    });
  });

  describe("isUser", () => {
    it("should return true when user is ROOT", () => {
      const userRoles = [ApplicationRoleEnum.root];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is ADMIN", () => {
      const userRoles = [ApplicationRoleEnum.admin];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is USER", () => {
      const userRoles = [ApplicationRoleEnum.user];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });

    it("should return false when user is GUEST", () => {
      const userRoles = [ApplicationRoleEnum.guest];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeFalsy();
    });

    it("should return false when user have no role", () => {
      const userRoles = [];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeFalsy();
    });

    it("should return true when user is [ROOT, ADMIN]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.user];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [GUEST, USER]", () => {
      const userRoles = [ApplicationRoleEnum.guest, ApplicationRoleEnum.user];
      expect(PermissionUtil.isUserPermission(userRoles)).toBeTruthy();
    });
  });

  describe("isNoPermission", () => {
    it("should return true when user is ROOT", () => {
      const userRoles = [ApplicationRoleEnum.root];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is ADMIN", () => {
      const userRoles = [ApplicationRoleEnum.admin];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is USER", () => {
      const userRoles = [ApplicationRoleEnum.user];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is GUEST", () => {
      const userRoles = [ApplicationRoleEnum.guest];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user have no role", () => {
      const userRoles = [];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, ADMIN]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ROOT, USER]", () => {
      const userRoles = [ApplicationRoleEnum.root, ApplicationRoleEnum.user];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [ADMIN, USER]", () => {
      const userRoles = [ApplicationRoleEnum.admin, ApplicationRoleEnum.user];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });

    it("should return true when user is [GUEST, USER]", () => {
      const userRoles = [ApplicationRoleEnum.guest, ApplicationRoleEnum.user];
      expect(PermissionUtil.isNoPermission(userRoles)).toBeTruthy();
    });
  });
});
